﻿// Windows version

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.Win32;

namespace List
{
    public partial class List : Form
    {
        public static int windowCount = 0;

        public List<int> fontSizes = null;

        public ContextMenu contextMenu;

        public List()
        {
            InitializeComponent();

            fontSizes = new List<int>();

            fontSizes.Add(8);
            fontSizes.Add(10);
            fontSizes.Add(12);
            fontSizes.Add(14);
            fontSizes.Add(16);
            fontSizes.Add(18);
            fontSizes.Add(20);
            fontSizes.Add(24);

            Setup();

            contextMenu = new ContextMenu();
            contextMenu.Popup += new EventHandler(CommonContextPopup);

            windowCount++;
        }

        private void ContextPopup(object sender, CancelEventArgs e)
        {
            CommonContextPopup(sender, e);
        }

        private void NewClicked(object sender, EventArgs e)
        {
            List list = new List();
            list.Show();
        }

        private void OpenClicked(object sender, EventArgs e)
        {
            DialogResult dr = openFileDialog.ShowDialog();

            if (dr.ToString() == "OK")
            {
                List list = new List();

                list.LoadFile(openFileDialog.FileName);

                if (list.treeView.Nodes.Count > 0)
                    list.treeView.SelectedNode = list.treeView.Nodes[0];

                list.UpdateTreeView(null);

                list.Show();
            }
        }

        private void CloseClicked(object sender, EventArgs e)
        {
            if (modified == true)
            {
                string label = "this list?";

                if (title.Length > 0)
                {
                    label = "\"" + title + "\"";
                }

                DialogResult result = MessageBox.Show("You have unsaved changes.  Are you sure" +
                    " you want to close " + label + "?",
                    "Comfirm close", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation,
                        MessageBoxDefaultButton.Button1);

                if (result.ToString() == "No")
                    return;
            }

            Dispose();
        }

        private void SaveClicked(object sender, EventArgs e)
        {
            CommonSaveClicked(sender, e);
        }

        private void SaveAsClicked(object sender, EventArgs e)
        {
            CommonSaveAsClicked(sender, e);
        }

        private void UncheckAllClicked(object sender, EventArgs e)
        {
            CommonUncheckAll(sender, e);
        }

        private void PreferencesClicked(object sender, EventArgs e)
        {
            CommonPreferencesClicked(sender, e);
        }

        private void ExitClicked(object sender, EventArgs e)
        {
            CommonExit(sender, e);
        }

        private void ExpandAllClicked(object sender, EventArgs e)
        {
            CommonExpandAllClicked(sender, e);
        }

        private void CollapseAllClicked(object sender, EventArgs e)
        {
            CommonCollapseAllClicked(sender, e);
        }

        private void ShowAllClicked(object sender, EventArgs e)
        {
            CommonShowAll(sender, e);
        }

        private void ShowTickedClicked(object sender, EventArgs e)
        {
            CommonShowTicked(sender, e);
        }

        private void ShowUntickedClicked(object sender, EventArgs e)
        {
            CommonShowUnticked(sender, e);
        }

        // Code retrieved from http://support.microsoft.com/kb/810001 - "How to display
        // a context menu that is specific to a selected TreeView node by using Visual
        // C# .NET or Visual C# 2005"

        private TreeNode m_OldSelectNode;

        private void treeView_MouseUp(object sender, MouseEventArgs e)
        {
            // Show menu only if the right mouse button is clicked.
            if (e.Button == MouseButtons.Right)
            {
                // Point where the mouse is clicked.
                Point p = new Point(e.X, e.Y);

                // Get the node that the user has clicked.
                TreeNode node = treeView.GetNodeAt(p);

                if (node != null)
                {
                    // Select the node the user has clicked.
                    // The node appears selected until the menu is displayed on the screen.
                    m_OldSelectNode = treeView.SelectedNode;
                    treeView.SelectedNode = node;
                    contextMenu.Show(treeView, p);
                }
            }
        }

        private void contextMenu_Closed(object sender, ToolStripDropDownClosedEventArgs e)
        {
            // Highlight the selected node.
            if (m_OldSelectNode != null)
            {
                //treeView.SelectedNode = m_OldSelectNode;
                m_OldSelectNode = null;
            }
        }

        private void titleBox_TextChanged(object sender, EventArgs e)
        {
            TitleBoxChanged(sender, e);
        }

        private void treeView_DoubleClick(object sender, EventArgs e)
        {
            EditClicked(sender, e);
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CommonAbout(sender, e);
        }

        // Label editing

        public Node quickModeNode = null;

        private void AfterLabelEdit(object sender, NodeLabelEditEventArgs e)
        {
            Node node = (Node)e.Node.Tag;

            // Remove this node if the label edit was cancelled and it was
            // empty to start with or if the label was emptied after the edit

            if (((e.Label == null) && (node.text.Length == 0)) ||
               ((e.Label != null) && (e.Label.Length == 0)))
            {

                SelectTreeNode(FindTreeNode(node).PrevVisibleNode);

                CommonUndo();
                UpdateTreeView(null);

                e.CancelEdit = true;
                return;
            }

            if (e.Label != null)
            {
                node.text = e.Label;
            }
        }

        private void StartQuickMode(Node parent, int index)
        {
            Node node = new Node("", this);

            foreach (string tag in tagFilter)
            {
                node.tags.Add(tag);
            }

            parent.Insert(index, node);
            parent.open = true;

            UpdateTreeView(null);

            quickModeNode = node;

            TreeNode newTreeNode = FindTreeNode(node);

            treeView.SelectedNode = newTreeNode;
            newTreeNode.EnsureVisible();
            newTreeNode.BeginEdit();

            treeView.Refresh();

            AddUndoFrame(this, UndoFrame.MODE_INSERT, node);
        }

        protected override bool ProcessCmdKey(ref Message msg, Keys keyData)
        {
            if (ActiveControl == treeView)
            {
                // Keys that apply when not editing a label

                if ((treeView.SelectedNode == null) || (treeView.SelectedNode.IsEditing == false))
                {
                    if (keyData == Keys.Insert)
                    {
                        Node quickNode;
                        int quickIndex;

                        if (treeView.SelectedNode != null)
                        {
                            Node node = (Node)treeView.SelectedNode.Tag;

                            quickNode = node.parent;
                            quickIndex = node.parent.items.IndexOf(node);
                        }
                        else
                        {
                            quickNode = visibleNode;
                            quickIndex = visibleNode.items.Count;
                        }

                        StartQuickMode(quickNode, quickIndex);
                        return true;
                    }

                    if (keyData == (Keys.Left | Keys.Control))
                    {
                        ScopeOut();
                        return true;
                    }

                    if (keyData == (Keys.Right | Keys.Control))
                    {
                        ScopeIn();
                        return true;
                    }

                    if (keyData == Keys.F2)
                    {
                        if (treeView.SelectedNode != null)
                        {
                            treeView.SelectedNode.BeginEdit();
                        }

                        return true;
                    }

                    if (keyData == Keys.Delete)
                    {
                        if (treeView.SelectedNode != null)
                        {
                            CommonDeleteItem((Node)treeView.SelectedNode.Tag);
                            return true;
                        }
                    }

                    if (keyData == (Keys.Control | Keys.X))
                    {
                        CommonCut();
                        return true;
                    }

                    if (keyData == (Keys.Control | Keys.C))
                    {
                        CommonCopy();
                        return true;
                    }

                    if (keyData == (Keys.Control | Keys.V))
                    {
                        CommonPaste();
                        return true;
                    }

                    if (keyData == (Keys.Control | Keys.Z))
                    {
                        CommonUndo();
                        return true;
                    }
                }

                if (keyData == Keys.Tab)
                {
                    IndentNode();
                    return true;
                }

                if (keyData == (Keys.Tab | Keys.Shift))
                {
                    UnindentNode();
                    return true;
                }

                // Keys for when a label is being edited

                if ((treeView.SelectedNode != null) && (treeView.SelectedNode.IsEditing == true))
                {
                    if ((keyData == Keys.Up) || (keyData == Keys.Down))
                    {
                        TreeNode target;

                        if (keyData == Keys.Up)
                        {
                            target = treeView.SelectedNode.PrevVisibleNode;
                        }
                        else
                        {
                            target = treeView.SelectedNode.NextVisibleNode;
                        }

                        treeView.SelectedNode.EndEdit(false);

                        if (target != null)
                        {
                            SelectTreeNode(target);
                            treeView.SelectedNode.BeginEdit();
                        }

                        return true;
                    }
                }

                if (keyData == Keys.Enter)
                {
                    if (treeView.SelectedNode == null)
                    {
                        StartQuickMode(visibleNode, visibleNode.items.Count);
                        return true;
                    }

                    Node node = (Node)treeView.SelectedNode.Tag;

                    if (treeView.SelectedNode.IsEditing)
                        treeView.SelectedNode.EndEdit(false);

                    if (node.parent != null)
                        StartQuickMode(node.parent, node.parent.items.IndexOf(node) + 1);

                    return true;
                }
            }

            return base.ProcessCmdKey(ref msg, keyData);
        }

        private void undoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CommonUndo();
        }

        private void showToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            SetupTagMenu();
        }

        private void EditMenu_DropDownOpening(object sender, EventArgs e)
        {
            CommonEditMenuOpening();
        }

        private void IndentNode()
        {
            TreeNode treeNode = treeView.SelectedNode;

            if (treeNode == null)
                return;

            bool wasEditing = treeNode.IsEditing;

            if (wasEditing)
            {
                treeNode.EndEdit(false);

                // Was the node deleted after the edit?

                if (treeView.SelectedNode != treeNode)
                {
                    return;
                }
            }

            Node node = ((Node)treeView.SelectedNode.Tag);
            int index = node.parent.items.IndexOf(node);

            if (index > 0)
            {
                // committed at this point

                AddUndoFrame(this, UndoFrame.MODE_MODIFY, node.parent);

                // If there is a previous sibling to this node, then we move the current node
                // to the end of the previous sibling's children

                Node target = node.parent.items[index - 1];

                target.Add(node);
                target.open = true;

                ListChanged();
                treeView.SelectedNode = FindTreeNode(node);

                if (wasEditing)
                {
                    treeView.SelectedNode.BeginEdit();
                }
            }
        }

        private void UnindentNode()
        {
            TreeNode treeNode = treeView.SelectedNode;

            if (treeNode == null)
                return;

            bool wasEditing = treeNode.IsEditing;

            if (wasEditing)
            {
                treeNode.EndEdit(false);

                // Was the node deleted after the edit?

                if (treeView.SelectedNode != treeNode)
                {
                    return;
                }
            }

            Node node = ((Node)treeView.SelectedNode.Tag);

            // can't unindent anything at root level

            if (node.parent == rootNode)
            {
                return;
            }

            // can't unindent out of the current view

            if (visibleNode != rootNode)
            {
                if (node.parent == visibleNode)
                {
                    return;
                }
            }

            // Going to do it

            List<Node> results = new List<Node>();

            FindAboveNodes(results, rootNode, node, 0);

            // Find new depth required

            int depth = 0;

            for (Node n = node; n != rootNode; n = n.parent, depth++) ;

            //MessageBox.Show("depth = " + depth);

            Node target = results[depth - 1];
            Node target_parent = target.parent;
            int target_position = target_parent.items.IndexOf(target);

            Node node_parent = node.parent;
            int node_position = node_parent.items.IndexOf(node);

            AddUndoFrame(this, UndoFrame.MODE_MODIFY, target_parent);

            // append all the next siblings as children of the node

            List<Node> nexts = new List<Node>();

            for (int i = node_position + 1; i < node_parent.items.Count; i++)
            {
                nexts.Add(node_parent.items[i]);
            }

            foreach (Node n in nexts)
            {
                node.Add(n);
            }

            if (nexts.Count > 0)
                node.open = true;

            target_parent.Insert(target_position + 1, node);

            modified = true;
            UpdateTreeView(null);

            treeView.SelectedNode = FindTreeNode(node);

            if (wasEditing)
            {
                treeView.SelectedNode.BeginEdit();
            }

            AutoSave();
        }

        // Tags

        private void TagFilterChecked(object sender, EventArgs e)
        {
            ToolStripMenuItem item = (ToolStripMenuItem)sender;
            String tag = item.Text;

            if ((item.Checked) && (tagFilter.Contains(tag) == false))
            {
                tagFilter.Add(tag);
            }
            else if ((item.Checked == false) && (tagFilter.Contains(tag) == true))
            {
                tagFilter.Remove(tag);
            }

            UpdateTagFilterVisibleState();
        }

        private void SetupTagMenu()
        {
            tagsMenuItem.DropDownItems.Clear();

            tagsMenuItem.Enabled = allTags.Count > 0;

            foreach (String tag in allTags)
            {
                ToolStripMenuItem item = new ToolStripMenuItem(tag);

                item.CheckOnClick = true;

                if (tagFilter.Contains(tag))
                {
                    item.Checked = true;
                }

                item.CheckedChanged += new EventHandler(TagFilterChecked);

                tagsMenuItem.DropDownItems.Add(item);
            }
        }

        // Drag and Drop

        private TreeNode dragNodeToSelect;
        private TreeNode srcTreeNode;
        private TreeNode targetTreeNode;
        private DRAG_MODE dragMode;
        private Rectangle dragRepairRect;

        private static int dropThreshold = 4;

        private enum DRAG_MODE { DRAG_NONE, DRAG_BEFORE, DRAG_INTO, DRAG_AFTER }

        private int debugOffset = 0;

        private void getDropTarget(object sender, DragEventArgs e)
        {
            srcTreeNode = null;
            targetTreeNode = null;

            if (e.Data.GetDataPresent("System.Windows.Forms.TreeNode", false))
            {
                Point pt = treeView.PointToClient(new Point(e.X, e.Y));

                srcTreeNode = (TreeNode)e.Data.GetData("System.Windows.Forms.TreeNode");
                targetTreeNode = treeView.GetNodeAt(pt);

                Node dragNode = (Node)srcTreeNode.Tag;
                Node targetNode = (Node)targetTreeNode.Tag;

                // ignore target if the dragged node is an ancestor

                for (Node node = targetNode; node != null; node = node.parent)
                {
                    if (node == dragNode)
                    {
                        targetTreeNode = null;
                        targetNode = null;
                    }
                }

                if (targetTreeNode != null)
                {
                    int offset = pt.Y - targetTreeNode.Bounds.Top;

                    if (offset <= dropThreshold)
                    {
                        dragMode = DRAG_MODE.DRAG_BEFORE;
                    }
                    else if (offset >= (targetTreeNode.Bounds.Height - dropThreshold))
                    {
                        dragMode = DRAG_MODE.DRAG_AFTER;
                    }
                    else
                    {
                        dragMode = DRAG_MODE.DRAG_INTO;
                    }

                    debugOffset = offset;
                }

                // if we're dragging after a node with at least one visible child, change
                // mode to drag the item before the first visible child

                if ((dragMode == DRAG_MODE.DRAG_AFTER) && (targetNode.VisibleCount > 0))
                {
                    dragMode = DRAG_MODE.DRAG_BEFORE;

                    foreach (Node child in targetNode.items)
                    {
                        if (child.visible)
                        {
                            targetNode = child;
                            targetTreeNode = FindTreeNode(targetNode);
                            break;
                        }
                    }
                }

                // ignore target if a node is dragged into its parent

                if ((dragMode == DRAG_MODE.DRAG_INTO) && (targetNode == dragNode.parent))
                {
                    dragMode = DRAG_MODE.DRAG_NONE;
                    targetTreeNode = null;
                    targetNode = null;
                }

                // ignore target if the node would end up where it started

                if ((dragMode == DRAG_MODE.DRAG_BEFORE) && (dragNode.parent == targetNode.parent) &&
                    (dragNode.parent.IndexOf(dragNode) == (targetNode.parent.IndexOf(targetNode) - 1)))
                {
                    dragMode = DRAG_MODE.DRAG_NONE;
                    targetTreeNode = null;
                    targetNode = null;
                }

                if ((dragMode == DRAG_MODE.DRAG_AFTER) && (dragNode.parent == targetNode.parent) &&
                    (dragNode.parent.IndexOf(dragNode) == (targetNode.parent.IndexOf(targetNode) + 1)))
                {
                    dragMode = DRAG_MODE.DRAG_NONE;
                    targetTreeNode = null;
                    targetNode = null;
                }
            }
        }

        private void repairDragInsertBar()
        {
            if (dragRepairRect != null)
            {
                treeView.Invalidate(dragRepairRect, true);
                treeView.Update();
            }
        }

        private void drawDragInsertBar(DRAG_MODE dragMode, TreeNode targetTreeNode, TreeNode dragTReeNode)
        {
            int left = 0;
            int top = 0;
            int width = 0;
            int height = 0;

            if (dragMode == DRAG_MODE.DRAG_BEFORE)
            {
                left = targetTreeNode.Bounds.Left;
                top = targetTreeNode.Bounds.Top - 1;
                width = targetTreeNode.Bounds.Width;
                height = 1;
            }
            else if (dragMode == DRAG_MODE.DRAG_AFTER)
            {
                left = targetTreeNode.Bounds.Left;
                top = (targetTreeNode.Bounds.Top + targetTreeNode.Bounds.Height) - 1;
                width = targetTreeNode.Bounds.Width;
                height = 1;
            }

            // special case where the nodes are adjacent visually and in the tree

            TreeNode beforeTreeNode;
            TreeNode afterTreeNode;

            if (dragMode == DRAG_MODE.DRAG_BEFORE)
            {
                beforeTreeNode = targetTreeNode.PrevVisibleNode;
                afterTreeNode = targetTreeNode;
            }
            else
            {
                beforeTreeNode = targetTreeNode;
                afterTreeNode = targetTreeNode.NextVisibleNode;
            }

            Node beforeNode = (Node)beforeTreeNode.Tag;
            Node afterNode = (Node)afterTreeNode.Tag;

            if (beforeNode.parent == afterNode.parent)
            {
                int beforeIndex = beforeNode.parent.IndexOf(beforeNode);
                int afterIndex = afterNode.parent.IndexOf(afterNode);

                if (beforeIndex == (afterIndex - 1))
                {
                    width = Math.Max(beforeTreeNode.Bounds.Width, afterTreeNode.Bounds.Width);
                }
            }

            // now draw it

            Graphics gfx = treeView.CreateGraphics();

            Pen pen = new Pen(Color.Gray);

            dragRepairRect = new Rectangle(left, top, width + 1, height + 2);

            gfx.DrawRectangle(pen, left, top, width, height);

            gfx.Dispose();
        }

        private void treeView_ItemDrag(object sender, ItemDragEventArgs e)
        {
            DoDragDrop(e.Item, DragDropEffects.Move);
        }

        private void treeView_DragEnter(object sender, DragEventArgs e)
        {
            dragNodeToSelect = treeView.SelectedNode;
            srcTreeNode = null;
            dragMode = DRAG_MODE.DRAG_NONE;

            e.Effect = DragDropEffects.Move;
        }

        private void treeView_DragOver(object sender, DragEventArgs e)
        {
            DRAG_MODE oldDragMode = dragMode;
            TreeNode oldTargetTreeNode = targetTreeNode;

            getDropTarget(sender, e);

            if (targetTreeNode != null)
            {
                e.Effect = DragDropEffects.Move;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }

            if ((dragMode != oldDragMode) || (targetTreeNode != oldTargetTreeNode))
            {
                repairDragInsertBar();

                if (dragMode == DRAG_MODE.DRAG_BEFORE)
                {
                    treeView.SelectedNode = null;

                    drawDragInsertBar(dragMode, targetTreeNode, srcTreeNode);
                }
                else if (dragMode == DRAG_MODE.DRAG_INTO)
                {
                    treeView.SelectedNode = targetTreeNode;
                }
                else if (dragMode == DRAG_MODE.DRAG_AFTER)
                {
                    treeView.SelectedNode = null;

                    drawDragInsertBar(dragMode, targetTreeNode, srcTreeNode);
                }
                else
                {
                    treeView.SelectedNode = null;
                }
            }
        }

        private void treeView_DragDrop(object sender, DragEventArgs e)
        {
            repairDragInsertBar();

            getDropTarget(sender, e);

            // We have to handle two cases here, one where the Drag and Drop operation
            // occurs within the same list and where it occurs between two distinct
            // lists

            Node srcNode = (Node)srcTreeNode.Tag;
            Node dstNode = (Node)targetTreeNode.Tag;

            List srcList = srcNode.list;
            List dstList = dstNode.list;

            Stack<UndoFrame> srcUndoFrames = new Stack<UndoFrame>();
            Stack<UndoFrame> dstUndoFrames = new Stack<UndoFrame>();

            srcUndoFrames.Push(CreateUndoFrame(this, UndoFrame.MODE_REMOVE, srcNode));
            srcNode.parent.Remove(srcNode);

            srcList.UpdateTreeView(null);

            if (dragMode == DRAG_MODE.DRAG_BEFORE)
            {
                dstNode.parent.Insert(dstNode.parent.IndexOf(dstNode), srcNode);
            }
            else if (dragMode == DRAG_MODE.DRAG_AFTER)
            {
                dstNode.parent.Insert(dstNode.parent.IndexOf(dstNode) + 1, srcNode);
            }
            else if (dragMode == DRAG_MODE.DRAG_INTO)
            {
                dstNode.Add(srcNode);

                if (dstNode.Count == 1)
                    dstNode.open = true;
            }

            if (srcNode.list == dstNode.list)
            {
                srcUndoFrames.Push(CreateUndoFrame(this, UndoFrame.MODE_INSERT, srcNode));
                srcList.AddUndoFrames(srcUndoFrames);
            }
            else
            {
                dstUndoFrames.Push(CreateUndoFrame(this, UndoFrame.MODE_INSERT, srcNode));
                srcList.AddUndoFrames(srcUndoFrames);
                dstList.AddUndoFrames(dstUndoFrames);

                srcNode.SetList(dstList);
            }

            dstList.UpdateTreeView(srcNode);
            //treeView.Refresh();

            dragNodeToSelect = dstList.FindTreeNode(srcNode);
        }

        private void treeView_DragLeave(object sender, EventArgs e)
        {
            repairDragInsertBar();

            treeView.SelectedNode = dragNodeToSelect;
        }

        private void List_FormClosed(object sender, FormClosedEventArgs e)
        {
            windowCount--;

            if (windowCount == 0)
            {
                Application.Exit();
            }
        }

        private void zoomBar_Scroll(object sender, EventArgs e)
        {
            prefs.fontSize = fontSizes[zoomBar.Value];
            prefs.SavePrefs();

            SetListFont();
        }
    }

    public class SpecificUI
    {
        public static void Clear(List list)
        {
            list.contextMenu.MenuItems.Clear();
        }

        public static void AddSeparator(List list, bool subMenu)
        {
            Add(list, "-", subMenu, true, null);
        }

        public static void Add(List list, string text, bool subMenu, bool enabled, EventHandler handler)
        {
            MenuItem item = new MenuItem();

            item.Text = text;
            item.Enabled = enabled;
            item.Click += handler;

            if (subMenu)
            {
                list.contextMenu.MenuItems[list.contextMenu.MenuItems.Count - 1].MenuItems.Add(item);
            }
            else
            {
                list.contextMenu.MenuItems.Add(item);
            }
        }

        public static void setStatusText(List list, string str)
        {
            list.statusLabel.Text = str;
        }
    }
}
