// Pocket PC version

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace List
{
    public partial class List : Form
    {
        public List()
        {
            InitializeComponent();
            Setup();
        }

        private void List_KeyDown(object sender, KeyEventArgs e)
        {
            if ((e.KeyCode == System.Windows.Forms.Keys.Enter))
            {
                treeView.SelectedNode.Checked = !treeView.SelectedNode.Checked;
            }
        }

        private void ContextPopup(object sender, EventArgs e) { CommonContextPopup(sender, e); }
        private void ExitClicked(object sender, EventArgs e) { CommonExit(sender, e); }
        private void PrefsClicked(object sender, EventArgs e) { CommonPreferencesClicked(sender, e); }
        private void UncheckAllClicked(object sender, EventArgs e) { CommonUncheckAll(sender, e); }
        private void SaveAsClicked(object sender, EventArgs e) { CommonSaveAsClicked(sender, e); }
        private void SaveClicked(object sender, EventArgs e) { CommonSaveClicked(sender, e); }
        private void AddClicked(object sender, EventArgs e) { CommonAdd(sender, e); }
        private void ExpandAllClicked(object sender, EventArgs e) { CommonExpandAllClicked(sender, e); }
        private void CollapseAllClicked(object sender, EventArgs e) { CommonCollapseAllClicked(sender, e); }
        private void ShowAllClicked(object sender, EventArgs e) { CommonShowAll(sender, e); }
        private void ShowTickedClicked(object sender, EventArgs e) { CommonShowTicked(sender, e); }
        private void ShowUntickedClicked(object sender, EventArgs e) { CommonShowUnticked(sender, e); }
        private void ItemChecked(object sender, TreeViewEventArgs e) { CommonItemChecked(sender, e); }
        private void GroupClosed(object sender, TreeViewEventArgs e) { CommonGroupClosed(sender, e); }
        private void GroupOpened(object sender, TreeViewEventArgs e) { CommonGroupOpened(sender, e); }

        private void NewClicked(object sender, EventArgs e)
        {
            if (modified == true)
            {
                DialogResult result = MessageBox.Show("You have unsaved changes.  Are you sure" +
                    " want to start a new list?",
                    "Comfirm new list", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation,
                        MessageBoxDefaultButton.Button1);

                if (result.ToString() == "No")
                    return;
            }

            NewList();
        }

        private void OpenClicked(object sender, EventArgs e)
        {
            if (modified == true)
            {
                DialogResult result = MessageBox.Show("You have unsaved changes.  Are you sure" +
                    " you want to open another list?",
                    "Comfirm open", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation,
                        MessageBoxDefaultButton.Button1);

                if (result.ToString() == "No")
                    return;
            }

            DialogResult dr = openFileDialog.ShowDialog();

            if (dr.ToString() == "OK")
            {
                LoadFile(openFileDialog.FileName);

                if (treeView.Nodes.Count > 0)
                    treeView.SelectedNode = treeView.Nodes[0];

                UpdateTreeView(null);
            }
        }

        private void titleBox_TextChanged(object sender, EventArgs e)
        {
            TitleBoxChanged(sender, e);
        }

        private void menuItem18_Click(object sender, EventArgs e)
        {
            CommonUndo();
        }

        private void tagsMenuItem_Popup(object sender, EventArgs e)
        {
            SetupTagMenu();
        }

        private void SetupTagMenu()
        {
            if (tagsMenuItem.MenuItems.Count > 0)
                return;

            tagsMenuItem.MenuItems.Clear();

            tagsMenuItem.Enabled = allTags.Count > 0;

            foreach (String tag in allTags)
            {
                MenuItem item = new MenuItem();

                item.Text = tag;

                if (tagFilter.Contains(tag))
                {
                    item.Checked = true;
                }

                item.Click += new EventHandler(TagFilterClicked);

                tagsMenuItem.MenuItems.Add(item);
            }
        }

        private void TagFilterClicked(object sender, EventArgs e)
        {
            MenuItem item = (MenuItem)sender;

            item.Checked = !item.Checked;

            String tag = item.Text;

            if ((item.Checked) && (tagFilter.Contains(tag) == false))
            {
                tagFilter.Add(tag);
            }
            else if ((item.Checked == false) && (tagFilter.Contains(tag) == true))
            {
                tagFilter.Remove(tag);
            }

            UpdateTagFilterVisibleState();

        }

        private void ShowMenu_Popup(object sender, EventArgs e)
        {
            SetupTagMenu();
        }
    }

    public class MItem : MenuItem
    {
    }

    public class SpecificUI
    {
        public static void Clear(List list)
        {
            list.contextMenu.MenuItems.Clear();
        }

        public static void AddSeparator(List list, bool subMenu)
        {
            Add(list, "-", subMenu, true, null);
        }

        public static void Add(List list, string text, bool subMenu, bool enabled, EventHandler handler)
        {
            MenuItem item = new MenuItem();

            item.Text = text;
            item.Enabled = enabled;
            item.Click += handler;

            if (subMenu)
            {
                list.contextMenu.MenuItems[list.contextMenu.MenuItems.Count - 1].MenuItems.Add(item);
            }
            else
            {
                list.contextMenu.MenuItems.Add(item);
            }
        }

        public static void setStatusText(List list, string str)
        {
            list.statusLabel.Text = str;
        }
    }
}
